#include <autoconf.h>
#include <param.h>
#include <string.h>
#include <herrno.h>
#include <scan.h>
#include <map.h>
#include <ar_search.h>

#define AR_GHEADER_MAGIC_NUM "!<arch>"
#define AR_GHEADER_MAGIC_NUM_SIZE 7

enum {
	UPDATE_FLAG_BOOT_OK,
	UPDATE_FLAG_START,
	UPDATE_FLAG_END,
	UPDATE_FLAG_FIRST_BOOTING,
};

static const char *update_flags[] = {
	[UPDATE_FLAG_BOOT_OK] = "UPDATE_FLAG_BOOT_OK",
	[UPDATE_FLAG_START] = "UPDATE_FLAG_START",
	[UPDATE_FLAG_END] = "UPDATE_FLAG_END",
	[UPDATE_FLAG_FIRST_BOOTING] = "UPDATE_FLAG_FIRST_BOOTING",
};

void get_update_flag(char flag[], u32 length)
{
	int ret;

	ret = get_param_value("@update_flag", flag, length);
	if (ret)
		strncpy(flag, update_flags[UPDATE_FLAG_BOOT_OK], length);

}

void set_update_flag(const char *flag)
{
	char *c_argv[2] = {"load_image", (char *)flag};
	char current_update_flag[32];

	get_update_flag(current_update_flag, sizeof(current_update_flag));

	if (strcmp(current_update_flag, flag) != 0)
		sethermit_param(2, c_argv, "@update_flag");
}

static void get_recovery_kernel_addr(addr_t recovery_start, size_t recovery_size,
				     addr_t *kernel_addr, size_t *kernel_size)
{
	 ar_search_item(recovery_start, recovery_size, 1, kernel_addr, kernel_size);
}

static void get_recovery_userland_addr(addr_t recovery_start, size_t recovery_size,
				       addr_t *userland_addr, size_t *userland_size)
{
	ar_search_item(recovery_start, recovery_size, 2, userland_addr, userland_size);
}

void get_region_addr_from_update_flag(const char *flag,
				      addr_t *kernel_start,
				      size_t *kernel_size,
				      addr_t *userland_start,
			              size_t *userland_size)
{
	if ((strcmp(flag, update_flags[UPDATE_FLAG_BOOT_OK]) == 0) ||
	    (strcmp(flag, update_flags[UPDATE_FLAG_END]) == 0)) {
		*kernel_start = get_flash_region_start_by_name("kernel");
		*kernel_size = get_flash_region_size_by_name("kernel");
		*userland_start = get_flash_region_start_by_name("userland");
		*userland_size = get_flash_region_size_by_name("userland");
	} else {
		hprintf("Recovery boot\n");
		get_recovery_kernel_addr(get_flash_region_start_by_name("recovery"),
					 get_flash_region_size_by_name("recovery"),
					 kernel_start,
					 kernel_size);
		get_recovery_userland_addr(get_flash_region_start_by_name("recovery"),
					    get_flash_region_size_by_name("recovery"),
					    userland_start,
					    userland_size);
	}
}

void get_next_update_flag(const char *currnt, const char **next)
{
	if (strcmp(currnt, update_flags[UPDATE_FLAG_END]) == 0)
		*next = update_flags[UPDATE_FLAG_FIRST_BOOTING];
	else
		*next = currnt;
}

#if defined(CONFIG_CMD_SETUPDATEFLAG)
static int setupdateflag_cmdfunc(int argc, char **argv)
{
	char flag[64];

	if (argc > 1) {
		int i;
		for (i = 0; i < ARRAY_SIZE(update_flags); i++) {
			if (strcmp(argv[1], update_flags[i]) == 0) {
				set_update_flag(argv[1]);
				return 0;
			}
		}

		hprintf("error: invalid parameter\n");
		hprintf("valid parameters\n");
		for (i = 0; i < ARRAY_SIZE(update_flags); i++)
			hprintf("   %s\n", update_flags[i]);

		return -H_EINVAL;
	} else {
		get_update_flag(flag, sizeof(flag));
		hprintf("%s\n", flag);
	}

	return 0;
}

const command_t setupdateflag_command =
	{ "setupdateflag", "<flag>",
	  "Set update-flag\n", &setupdateflag_cmdfunc };

COMMAND_ABBR(setupdateflag_command, 'u');
#endif
